<?php

namespace Drupal\markdown_exporter\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Url;
use Drupal\Component\Utility\Crypt;
use Drupal\Component\Serialization\Json;

/**
 * Class MarkdownExporterController.
 *
 * @package Drupal\markdown_exporter\Controller
 */
class MarkdownExporterController extends ControllerBase {
  /**
   * Reload the previous page.
   */
  public function downloadFile() {
    $request = $this->requestStack->getCurrentRequest();
    $settings = \Drupal::config('markdown_exporter.settings');
    $md_dir = 'public://' . $settings->get('markdown_exporter_dir');
    $zip_file = $settings->get('markdown_exporter_dir', 'markdown_export') . '_' . REQUEST_TIME . '.zip';
    $zip_uri = 'temporary://' . $zip_file;

    $zip = new \ZipArchive();
    if ($handle = opendir($md_dir)) {
      $result = $zip->open(drupal_realpath($zip_uri), \ZipArchive::CREATE);
      if (!$result) {
        throw new DriverException('Zip archive could not be created. Error ' . $result);
      }

      while (false !== ($file_name = readdir($handle))) {
        if ($file_name != '.' && $file_name != '..') {
            $zip->addFile(drupal_realpath($md_dir).'/'.$file_name, $file_name);
        }
      }
      closedir($handle);
      $result = $zip->close();
      if (!$result) {
        throw new DriverException('Zip archive could not be closed.');
      }
    }
    $response = new Response();
    $response->headers->set('Content-Type', 'text/html; charset=utf-8');
    $response->header->set("Cache-Control: public");
    $response->header->set("Content-Type: application/zip");
    $response->header->set("Cache-Control: no-store, no-cache, must-revalidate");
    $response->header->set("Cache-Control: post-check=0, pre-check=0");
    $response->header->set("Content-Disposition: attachment; filename=\"$zip_file\";");
    $response->header->set("Content-Transfer-Encoding: binary");

    $fp = fopen(drupal_realpath($zip_uri), 'rb');
    fpassthru($fp);

    return $response;
  }

  /**
   * Authorize with GitHub to generate access token and redirect
   *
   * @return void
   */
  public function githubReauthorize () {
    $markdown_exporter_config = \Drupal::config('markdown_exporter.settings');
    $markdown_exporter_config_editable = \Drupal::configFactory()->getEditable('markdown_exporter.settings');
    $queryparams = \Drupal::request()->query->get('code');
    if($queryparams) {
      /* need to swap this code for a permanent access_token */
      $url = 'https://github.com/login/oauth/access_token';
      $headers = ['Accept' => 'application/json'];
      $params = [
        'client_id'=> $markdown_exporter_config->get('github_oauth_app_clientid'),
        'client_secret'=> $markdown_exporter_config->get('github_oauth_app_clientsecret'),
        'code' => $queryparams,
        'state' => $markdown_exporter_config->get('github_state_var')
      ];
      $result = \Drupal::httpClient()->post($url, [
        'form_params' => $params,
        // 'body' => http_build_query($params),
        'headers' => $headers,
        'http_errors' => FALSE,
      ]);
      $resp = $result->getBody()->getContents();
      $response = Json::decode($resp);
      /* store the authorised token */
      if(array_key_exists('access_token', $response)) {
        $markdown_exporter_config_editable
        ->set('github_access_token', $response['access_token'])
        ->save();
        /* add a success message */
        \Drupal::messenger()->addMessage(t('GitHub authorization complete.'), 'status');
      } else {
        /* add a success message */
        \Drupal::messenger()->addMessage(t('Already authenticated with GitHub.'), 'status');
      }

      return $this->redirect('markdown_exporter.github_auth');
    }
  } 
}