<?php

namespace Drupal\markdown_exporter\EventSubscriber;

use Symfony\Component\DependencyInjection\ContainerAwareInterface;
use Symfony\Component\DependencyInjection\ContainerAwareTrait;
use Drupal\Component\Serialization\Json;
use Drupal\Core\File\FileSystemInterface;
use Drupal\Core\Url;
use Drupal\file\Entity\File;
use Markdownify\Parser;
use Markdownify\Converter;

class MarkdownExporterGenerator implements ContainerAwareInterface {

	use ContainerAwareTrait;	
	
	/**
  * Creating content files in MD format
  *
	* Converts the content of the specified types to markdown,
	* and creates a zip containing them.
	* @param array $get_types
	* @param string $mode
  */
	public function bulkExport ($get_types, $mode = '')
	{
		$settings = \Drupal::config('markdown_exporter.settings');
		$md_dir = 'public://' . $settings->get('markdown_exporter_dir');
		$field_config = $settings->get('markdown_exporter_fields');
		$success = true;

		\Drupal::service('file_system')->prepareDirectory($md_dir, FileSystemInterface::CREATE_DIRECTORY | FileSystemInterface::MODIFY_PERMISSIONS);

		$mdObj = new Converter(0, MDFY_BODYWIDTH, MDFY_KEEPHTML);
		
		$types = array_values($get_types);
		array_pop($types);
		$query = \Drupal::entityQuery('node');
    $query
      ->condition('type', $types, 'IN')
      ->condition('status', 1)
			->sort('created', 'ASC');

		$nids = $query->execute();
		if (count($nids)) {    
			$node_details = \Drupal\node\Entity\Node::loadMultiple($nids);   
		}
		
		$entity_fields = Json::decode($field_config);
		
		
		$files = array();
		if(!empty($node_details)) {
			foreach ($node_details as $node) {
				$url_alias = \Drupal::service('path.alias_manager')->getAliasByPath('/node/'.$node->id());
				
				// formatting the alias
				$url_alias_formatted = strpos($url_alias, '/') ? str_replace('/', "-", $url_alias) : $url_alias;
				 
				// Content file in .md format
				$file_name = $url_alias_formatted . '.md';

				$contents = \Drupal::service('markdown_exporter.marekdown_helper')->mdContentGenerate($url_alias, $node, $entity_fields);

				// and create the file
				$replace = ($settings->get('markdown_file_overwrite') == TRUE) ?
				FILE_EXISTS_REPLACE : FILE_EXISTS_RENAME;
				if($success) {
					drupal_chmod($md_dir);
					$base_file = $md_dir . $file_name;
					$result = \Drupal::service('file_system')->saveData($contents, $base_file, FileSystemInterface::EXISTS_REPLACE);
					\Drupal::service('markdown_exporter.marekdown_helper')->pushToGitRepo($base_file);
				}
			}
			$msg = ($mode == 'cli') ? t('MarkDown files are generated & saved in')
			 . ' files/'. variable_get('markdown_exporter_dir', 'markdown_export')
				: $msg = t('MarkDown files are generated!') . ' '
				 . t(
					 '<a href=":url">Click </a> to download the files in ziped format',
					  [':url' => Url::fromRoute('markdown_exporter.download')->toString()]
					);
			
			\Drupal::messenger()->addMessage($msg, 'status');
			
			\Drupal::messenger()->addMessage("Files committed to repository", 'status');
		}

		return false;
	}
	
	/**
   * Creating single node content files in MD format
   *
	 * Converts the content of the node to markdown
	 * @param Drupal\Core\Entity\EntityInterface $entity
	 * @param string $mode
   */
	public function generateMarkdownNodeContent ($entity, $mode = NULL) {
		$markdown_exporter_config = \Drupal::config('markdown_exporter.settings');
		$markdown_exporter_fields = Json::decode($markdown_exporter_config->get('markdown_exporter_fields'));
		
		if (!array_key_exists($entity->bundle(), $markdown_exporter_fields) && empty($markdown_exporter_fields[$entity->bundle()])) {
			return;
		}

		$md_dir = 'public://' . $markdown_exporter_config->get('markdown_exporter_dir');
		$success = true;

		\Drupal::service('file_system')->prepareDirectory($md_dir, FileSystemInterface::CREATE_DIRECTORY | FileSystemInterface::MODIFY_PERMISSIONS);

		$url_alias = \Drupal::service('path.alias_manager')->getAliasByPath('/node/'.$entity->id());
		
		// formatting the alias
		$url_alias_formatted = strpos($url_alias, '/') ? str_replace('/', "-", $url_alias) : $url_alias;
			
		// Content file in .md format
		$file_name = $url_alias_formatted . '.md';

		$contents = \Drupal::service('markdown_exporter.marekdown_helper')->mdContentGenerate($url_alias, $entity, $markdown_exporter_fields);

		$replace = ($markdown_exporter_config->get('markdown_file_overwrite') == TRUE) ?
				FILE_EXISTS_REPLACE : FILE_EXISTS_RENAME;
		if($success) {
			drupal_chmod($md_dir);
			$base_file = $md_dir . $file_name;
			$result = \Drupal::service('file_system')->saveData(
				$contents, $base_file, FileSystemInterface::EXISTS_REPLACE
			);
			$file = File::create([
				'uri' => $base_file,
				'uid' => \Drupal::currentUser()->id(),
				'status' => FILE_STATUS_PERMANENT,
			]);
			if (NULL !== $file->getFileUri()) {
				$msg = ($mode == 'cli') ? t('MarkDown files are generated & saved in')
			 . ' files/'. variable_get('markdown_exporter_dir', 'markdown_export')
				: $msg = t('MarkDown file has been generated!');

				\Drupal::messenger()->addMessage($msg, 'status');
				\Drupal::service('markdown_exporter.marekdown_helper')->pushToGitRepo($file_name, $file, $entity->id());
				\Drupal::messenger()->addMessage("Files committed to repository", 'status');
			} else {
				\Drupal::messenger()->addMessage(t('Error in generating markdown'), 'status');
			}
		}
	}
}
